// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
			 "cloudera/chart/TimeRange",
			 "cloudera/chart/TimeBrowserServerState",
			 "cloudera/Util"
], function(TimeRange, TimeBrowserServerState, Util) {

	// Responsibilities:
	// * Updates the URL hash when the state object changes.
	// * Updates the state object when the URL hash changes.
	//
	// Note: TimeBrowserServerState is a serializable form
	// of a TimeBrowserState object.
	return Class.extend({

		init: function(options, state) {
			this.state = state;
			this.initializeURLChanger();
			// we dont want the very first localServerState to update.
			this.localServerState = new TimeBrowserServerState(this.state);

			// Don't run the update loop in test mode.
			if (!Util.getTestMode()) {
				// Don't call this.update too many times.
				jQuery.subscribe("timeBrowserStateChanged",
												 _.debounce(_.bind(this.update, this), 1000));
			}
		},

		// Listens for URL hash change events.
		initializeURLChanger: function() {
			var self = this;
			$(window).hashchange( function() {
				if (!Util.getTestMode()) {
					var urlParams = location.hash.replace(/^#/, '');
					self.updateFromURLParams(urlParams);
				}
			});
		},

		// Listens for state change events.
		update: function() {
			var self = this;
			var newState = new TimeBrowserServerState(this.state);
			if (!self.localServerState.equals(newState)) {
				self.localServerState = newState;
				self.updateHash(self.localServerState);
			}
		},

		// The URL has changed, notify the actual state object.
		updateFromURLParams : function(urlParams) {
			var serverState = this.getServerStateFromURLParams(urlParams);
			this.state.setFromServerState(serverState);
		},

		// Converts the URL parameters into a server state.
		getServerStateFromURLParams: function(urlParams) {
			var params = Util.unparam(urlParams);
			var serverState = new TimeBrowserServerState(this.state);
			serverState.fromParams(params);
			return serverState;
		},

		// Updates the URL hash from the state object.
		updateHash: function(serverState) {
			if (!Util.getTestMode()) {
				location.hash = this.toURLParams(serverState, location.hash.substring(1));
			}
		},

		// Converts the serverState to a new URL hash.
		toURLParams: function(serverState, urlParams) {
			// Use the existing urlParams as the starting point in case
			// there are parameters unknown to this component.
			var result = Util.unparam(urlParams);
			$.extend(result, serverState.toParams());
			return $.param(result);
		}
	});
});
