// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
			 "cloudera/Util",
			 "cloudera/chart/TimeRange",
			 "cloudera/chart/TimeBrowserState",
			 "cloudera/chart/TimeBrowserServerState"
], function(Util, TimeRange, TimeBrowserState, TimeBrowserServerState) {

	describe("TimeBrowserServerState Tests", function() {
		var now;

		beforeEach(function() {
			var options = {
				isCurrentMode : false,
				firstVisibleDate : new Date(1),
				lastVisibleDate : new Date(10),
				markerDate : new Date(5),
				firstDate : new Date(1),
				lastDate : new Date(20)
			};
			now = new Date(20);
			spyOn(Util, "getServerNow").andReturn(now);

			state = new TimeBrowserState(options);
		});

		afterEach(function() {
		});

		it("should create a basic server state.", function() {
			var serverState = new TimeBrowserServerState(state);

			expect(serverState.lastTime).toEqual(20);
			expect(serverState.firstTime).toEqual(1);
			expect(serverState.markerTime).toEqual(5);
			expect(serverState.startTime).toEqual(1);
			expect(serverState.endTime).toEqual(10);
		});

		it("should create another server state in the future but they should be considered identical.", function() {
			state.setCurrentMode(true);
			state.setSelectedRange(new TimeRange(new Date(11), new Date(20)));
			state.setMarkerDate(new Date(20));
			var serverState = new TimeBrowserServerState(state);

			now = new Date(30);
			state.moveToNow();
			var serverState1 = new TimeBrowserServerState(state);

			expect(serverState.equals(serverState1)).toBeTruthy();
		});

		it("should create another server state in the past and they should be considered identical.", function() {
			now = new Date(20);
			var serverState = new TimeBrowserServerState(state);
			var serverState1 = new TimeBrowserServerState(state);

			expect(serverState.equals(serverState1)).toBeTruthy();
		});

		it("should create another server state in the past with a different marker date, and they should not be considered identical.", function() {
			var serverState = new TimeBrowserServerState(state);
			state.setMarkerDate(new Date(6));
			var serverState1 = new TimeBrowserServerState(state);
			expect(serverState.equals(serverState1)).toBeFalsy();
		});

		it("should call toParams and fromParams", function() {
			now = new Date(20);
			var serverState = new TimeBrowserServerState(state);

			state.setMarkerDate(new Date(6));
			state.setSelectedRange(new TimeRange(new Date(5), new Date(20)));

			var serverState1 = new TimeBrowserServerState(state);
			serverState.fromParams(serverState1.toParams());

			expect(serverState.equals(serverState1)).toBeTruthy();
		});

		it("should handle empty urlParams object", function() {
			now = new Date(20);
			var serverState = new TimeBrowserServerState(state);
			var serverState1 = new TimeBrowserServerState(state);
			serverState.fromParams({});
			expect(serverState.equals(serverState1)).toBeTruthy();
		});
	});
});
