// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/cmf/include/DataTableColumnRenderer",
	"cloudera/cmf/include/HostRoleColumnRenderer",
	"cloudera/cmf/include/DataTableUtil",
	"cloudera/Util"
], function(DataTableColumnRenderer, HostRoleColumnRenderer, DataTableUtil, Util) {

	return function(options) {
		var tableId = options.tableId;
		var THROTTLE_TIMEOUT_MS = Math.max(options.assignments.length / 4, 50);
		var i = 0;
		var HOST_COL = i++;
		var HOST_IP_COL = i++;
		var HOST_RACK_COL = i++;
		var HOST_HEALTH_COL = i++;
		var ANN_COL = i++;
		var SBN_COL = i++;
		var JN_COL = i++;
		var SNN_COL = i++;
		var NS_COL = i++;
		var ROLE_COL = i++;
		var ROLE_COUNT_COL = i++;

		var invisibleColumns = [ROLE_COUNT_COL];
		if (!options.showANN) {
			invisibleColumns.push(ANN_COL);
		}

		if (!options.showSBN) {
			invisibleColumns.push(SBN_COL);
		}

		if (!options.showJN) {
			invisibleColumns.push(JN_COL);
		}

		if (!options.showSNN) {
			invisibleColumns.push(SNN_COL);
		}

		var dataTableSettings = {
			"aaData": options.assignments,
			"bInfo": false,
			"bSortCellsTop": true,
			"bAutoWidth": false,
			"bDeferRender": true,
			"oLanguage": {
				"sLengthMenu": DataTableColumnRenderer.getPaginationMenu()
			},
			"aaSorting": [[ HOST_COL, "asc" ]],
			"aoColumnDefs": [
				{ "bSortable": false, "aTargets": [ANN_COL, SBN_COL, JN_COL,  SNN_COL] },
				{ "bVisible": false, "aTargets": invisibleColumns },
				{ "iDataSort": ROLE_COUNT_COL, "aTargets": [ROLE_COL] },
				{ "sClass": "alignCenter cellSelectable", "aTargets": [ANN_COL, SBN_COL, JN_COL, SNN_COL] },
				{ "aTargets": [HOST_COL], "fnRender": function(oObj) { return DataTableColumnRenderer.renderHost(oObj.aData.host); }},
				{ "aTargets": [HOST_IP_COL], "fnRender": function(oObj) { return DataTableColumnRenderer.renderIP(oObj.aData.host); }},
				{ "aTargets": [HOST_RACK_COL], "fnRender": function(oObj) { return DataTableColumnRenderer.renderRack(oObj.aData.host); }},
				{ "aTargets": [HOST_HEALTH_COL], "fnRender": function(oObj) { return DataTableColumnRenderer.renderHealth(oObj.aData.host); }},
				{ "aTargets": [ANN_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderANN(oObj.aData.host, options.nameservice, options.enableHA); }},
				{ "aTargets": [SBN_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderSBN(oObj.aData.host, options.nameservice, options.enableHA); }},
				{ "aTargets": [JN_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderJN(oObj.aData.host, options.nameservice, options.enableHA); }},
				{ "aTargets": [SNN_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderSNN(oObj.aData.host, options.nameservice, options.enableHA); }},
				{ "aTargets": [NS_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderNameservices(oObj.aData.host); }},
				{ "aTargets": [ROLE_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderRoles(oObj.aData.host); }},
				{ "aTargets": [ROLE_COUNT_COL], "fnRender": function(oObj) { return HostRoleColumnRenderer.renderRoleCount(oObj.aData.host); }}
			]
		};

		var MIN_PAGINATION_LENGTH = 10;
		var enablePagination = options.assignments.length > MIN_PAGINATION_LENGTH;
		var table = DataTableUtil.initialize(tableId, dataTableSettings, enablePagination, options);

		var match = DataTableColumnRenderer.match;
		var matchCell = DataTableColumnRenderer.matchCell;
		var matchDropdown = DataTableColumnRenderer.matchDropdown;

		var $filterName = $("#filterName");
		var $filterIp = $("#filterIp");
		var $filterRack = $("#filterRack");
		var $filterHostHealth = $("#filterHostHealth");
		var $filterNameservices = $("#filterNameservices");
		var $filterOtherRoles = $("#filterOtherRoles");

		//Handles the logic for row filtering.  Returns
		//true if a row passes all the filters and
		//should be displayed.
		$.fn.dataTableExt.afnFiltering.push(function( oSettings, aData, iDataIndex ) {
			if ( $(oSettings.nTable).attr("id") !== tableId ) {
				return true;
			}

			var result = true;
			// filter is turned off.
			if (options.filter !== undefined && !options.filter) {
				return result;
			}

			var notExact = false;

			result = matchCell(aData, HOST_COL, $filterName, notExact) &&
				matchCell(aData, HOST_IP_COL, $filterIp, notExact) &&
				matchCell(aData, HOST_RACK_COL, $filterRack, notExact) &&
				matchDropdown(aData, HOST_HEALTH_COL, $filterHostHealth, notExact) &&
				matchCell(aData, NS_COL, $filterNameservices, notExact) &&
				matchCell(aData, ROLE_COL, $filterOtherRoles, notExact);
			return result;
		});

		var redrawTable = function() {
			$("#" + tableId).dataTable().fnDraw();
		};

		var throttledRedraw = Util.throttle(redrawTable, THROTTLE_TIMEOUT_MS);
		$("input.filter").keyup(throttledRedraw);
		$(".filter").change(throttledRedraw);

		var toggleHoverClass = function(evt, isAdd) {
			var $target = $(evt.target);
			if (!$target.is(".cellSelectable")) {
				$target = $target.closest("td");
			}
			if ($target.find("input").length > 0) {
				if (isAdd) {
					$target.addClass("hoverable");
				} else {
					$target.removeClass("hoverable");
				}
			}
		};

		var clickInput = function(evt) {
			var $input, $target = $(evt.target);
			if ($target.is("input")) {
				return;
			}
			$input = $target.find("input");
			$input.trigger("click");
		};

		var publishSelectHostAssignment = function(evt) {
			var $input = $(evt.target);
			$.publish("selectHostAssignment", [$input.attr("name"), $input.attr("value"), $input.is(':checked')]);
		};

		$("#" + tableId).on('click', "td.cellSelectable", function(evt) {
			clickInput(evt);
		});

		$("#" + tableId).on('change', "td.cellSelectable input", function(evt) {
			publishSelectHostAssignment(evt);
		});

		$("#" + tableId).on('mouseover', "td.cellSelectable", function(evt) {
			toggleHoverClass(evt, true);
		});

		$("#" + tableId).on('mouseout', "td.cellSelectable", function(evt) {
			toggleHoverClass(evt, false);
		});
	};
});
