// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/cmf/page/ServiceChartSet",
	"cloudera/cmf/page/ServiceMetricSelector",
	"cloudera/Util"
], function(ServiceChartSet, ServiceMetricSelector, Util) {

return function(options) {

	// The maximum number of charts we can show initially.
	var maxInitialChartCount = 2;

	/**
	 * Get the total number of distinct charts.
	 */
	var getChartCount = function(timeSeriesDescriptors) {
		var chartIds = {};
		$.each(timeSeriesDescriptors, function(index, tsd) {
			chartIds[tsd.chart.id] = 1;
		});

		var chartCount = 0;
		$.each(chartIds, function(key, value) {
			chartCount += 1;
		});
		return chartCount;
	};

 /**
	 * @return those time series descriptors that have the chart Ids
	 * in the selectedChartIds list.
	 */
	var filterTimeSeriesDescriptors = function(timeSeriesDescriptors,
		selectedChartIds) {
		var i, j, result = [];

		for (i = 0; i < timeSeriesDescriptors.length; i += 1) {
			var tsd = timeSeriesDescriptors[i];
			var chartId = tsd.chart.id;
			var found = false;
			for (j = 0; j < selectedChartIds.length; j += 1) {
				var selectedChartId = selectedChartIds[j];
				if (selectedChartId === chartId) {
					found = true;
					break;
				}
			}
			if (found) {
				result.push(tsd);
			}
		}
		return result;
 };

 var filteredDescriptors = filterTimeSeriesDescriptors(options.timeSeriesDescriptors,
		options.selectedChartIds);

	var params = {
		'urlParams': options.urlParams,
		'startDate': options.startDate,
		'endDate': options.endDate,
		'timeSeriesDescriptors': filteredDescriptors,
		'jsonURL': options.jsonURL,
		'chartWidth': options.width,
		'chartHeight': options.height,
		'width': options.width,
		'height': String(options.height * getChartCount(filteredDescriptors)) + "px"
	};

	var serviceChartSet = new ServiceChartSet(params, options.id);

	/**
	 * Add all the time series.
	 */
	var process = function(timeSeriesDescriptors) {
		if (timeSeriesDescriptors.length > 0) {
			$.each(timeSeriesDescriptors, function(index, tsd) {
				serviceChartSet.addTimeSeries(tsd);
				serviceChartSet.showChart(tsd.chart.id, 0);
				serviceChartSet.showTimeSeries(tsd);
			});
			serviceChartSet.render();
		}
	};

	var handleHealthCheckResponse = function(response) {
		process(Util.filterError(response));
	};

	var onHealthCheckSelectionChanged = function(testName, key) {
		var params = {
			testName: testName
		};
		if (key) {
			params.key = key;
		}
		$.get(options.healthCheckTSAjaxURL + "?" + $.param(params), handleHealthCheckResponse);
	};
	jQuery.subscribe("healthCheckSelectionChanged", onHealthCheckSelectionChanged);

	var opts = {
		'addButtonId': options.addButtonId,
		'entries': [],
		'updateUrl': options.updateUrl,
		'dialogWidth': options.dialogWidth,
		'dialogHeight': options.dialogHeight,
		'dialogTitle': options.dialogTitle,
		'descriptionTitle': options.descriptionTitle,
		'serviceChartSet': serviceChartSet,
		'timeSeriesDescriptors': options.timeSeriesDescriptors
	};
	var serviceMetricSelector = new ServiceMetricSelector(opts);
};

});
