// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util",
	"cloudera/headlamp/hdfs/include/FileSearchResultTable"
], function(Util, FileSearchResultTable) {

	/**
	 * Listens for "fileSearchFilterChanged" event,
	 * and refetches the table.
	 *
	 * @param options {
	 *     searchUrl: "...",
	 *     tableId: "...",
	 *     path: "...",
	 *     queryTerms: {
	 *         data: [{
	 *             // see QueryTerm.js
	 *         }, {
	 *             // see QueryTerm.js
	 *         }]
	 *     },
	 *     watchedDirsManager: // something to add or remove watched dirs.
	 * }
	 */
	return function(options) {
		var initialized = false;
		var resultTable = new FileSearchResultTable(options);

		var populateData = function(response) {
			$(".fetchingData").addClass("hidden");
			// the response corresponds to a JSON array.
			// Each row is represented by the FileSearchResult Java Class.
			var filteredResponse = Util.filterJsonResponseError(response);
			if (filteredResponse && filteredResponse.results) {
				resultTable.render(filteredResponse.results, initialized);
				initialized = true;
			}
		};

		var onSearchFilterChanged = function(searchUrl, searchJson, searchMode) {
			var params = {
				json: searchJson
			};
			if (searchMode) {
				$(".searchResultsHeader").removeClass("hidden");
			} else {
				$(".searchResultsHeader").addClass("hidden");
			}
			$(".fetchingData").removeClass("hidden");
			$.post(searchUrl, params, populateData);
		};

		// initial query on page load.
		if (options.queryTerms) {
			onSearchFilterChanged(options.searchUrl, JSON.stringify(options.queryTerms, options.searchMode));
		}

		jQuery.subscribe("fileSearchFilterChanged", onSearchFilterChanged);
	};
});
