// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
/**
 * jQuery's implementation of toggling the display of another element.
 */
define([
	"cloudera/layout/ElementFinder"
], function(ElementFinder) {

	var Toggler = function (element) {
		this.$element = $(element);

		// Example 1:
		// <a href="#" data-element-direction="next" data-element-selector="ul">
		//   <i class="arrow Icon arrowRight"></i>Some Link that would toggle the ul
		// </a>
		// <ul style="display:none">
		//    <li>...</li>
		// </ul>
	  // displays:
		// > Some Link that would toggle the ul
		//
		// Example 2:
		// <a href="#" data-element-direction="next" data-element-selector="ul" data-arrow-position="right">
		//   Some Link that would toggle the ul. <i class="arrow Icon arrowDown"></i>
		// </a>
		// <ul>
		//    <li>...</li>
		// </ul>
	  // displays:
		// Some Link that would toggle the ul v
		//
		// Example 3:
		// <a href="#" data-element-direction="next" data-element-selector="ul" data-arrow-position="right">
		//   Some Link that would toggle the ul. <i class="arrow Icon arrowLeft"></i>
		// </a>
		// <ul style="display:none">
		//    <li>...</li>
		// </ul>
	  // displays:
		// Some Link that would toggle the ul <
		this.onclick = function (evt) {
			var $elem = this.$element;

			// data-arrow-position="left|right"
			// if the expand/collapse icon should be placed on the left,
			// then the expand icon should be pointing to the opposite direction: right.
			var arrowPosition = $elem.attr("data-arrow-position") || "left";
			var arrowClass = "arrowRight";

			if (arrowPosition === "right") {
				arrowClass = "arrowLeft";
			}

			// use ElementFinder to identify the target.
			// data-element-direction="..."
			// data-element-selector="..."
			var $target = $elem.ElementFinder();
			if ($target) {
				if ($target.is(":visible")) {
					$elem.find(".arrow").removeClass("arrowDown").addClass(arrowClass);
					$target.hide();
				} else {
					$elem.find(".arrow").removeClass(arrowClass).addClass("arrowDown");
					$target.show();
				}
			}

			// publish an event on toggle.  see OPSAPS-7419.
			$.publish("toggle", [$elem]);

			if (evt) {
				evt.preventDefault();
			}
		};
	};

	$.fn.Toggler = function (option) {
		return this.each(function () {
			var $this = $(this),
			data = $this.data('Toggler'),
			onclick = function (evt) {
				data.onclick(evt);
			};

			if (!data) {
				$this.data('Toggler', (data = new Toggler(this)));
			}

			if (option === 'click') {
				data.onclick(null);
			}
		});
	};

	return $.fn.Toggler;
});
